import { InjectionToken, Directive, Inject, Input, NgModule } from '@angular/core';
import { ControlContainer } from '@angular/forms';
import { Router } from '@angular/router';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import set from 'lodash.set';

function parse(value, type) {
    switch (type) {
        case 'string':
            return value;
        case 'boolean':
            return value === 'false' ? false : !!value;
        case 'array':
            return value === null || value === void 0 ? void 0 : value.split(',');
        case 'number':
            return +value;
        default:
            return value;
    }
}
function get(obj, path) {
    let current = obj;
    path.split('.').forEach((p) => (current = current[p]));
    return current;
}
function resolveParams(defs, formValue, group) {
    const result = {};
    defs.forEach((def) => {
        const dirty = group.get(def.path).dirty;
        if (dirty) {
            const value = get(formValue, def.path);
            result[def.queryKey] = value || typeof value === 'boolean' ? value.toString() : null;
        }
    });
    return result;
}

class QueryParamDef {
    constructor(config) {
        this.config = config;
    }
    get queryKey() {
        return this.config.queryKey;
    }
    get path() {
        return this.config.path || this.queryKey;
    }
    get type() {
        return this.config.type || 'string';
    }
    get trigger() {
        return this.config.trigger || 'change';
    }
    get strategy() {
        return this.config.strategy || 'twoWay';
    }
    parse(queryParamValue) {
        return parse(queryParamValue, this.type);
    }
}
class BindQueryParamsManager {
    constructor(defs) {
        this.defs = defs.map((def) => new QueryParamDef(def));
    }
    getDef(queryKey) {
        return this.defs.find((def) => def.queryKey === queryKey);
    }
    parse(queryParams) {
        const result = {};
        for (const [key, value] of Object.entries(queryParams)) {
            result[key] = this.getDef(key).parse(value);
        }
        return result;
    }
}
const BIND_QUERY_PARAMS_OPTIONS = new InjectionToken('BIND_QUERY_PARAMS_OPTIONS', {
    providedIn: 'root',
    factory() {
        return {
            windowRef: window,
        };
    },
});

class BindQueryParamsDirective {
    constructor(formGroupDirective, router, options) {
        this.formGroupDirective = formGroupDirective;
        this.router = router;
        this.options = options;
        this.destroy = new Subject();
    }
    get defs() {
        return this._defs.defs;
    }
    ngOnInit() {
        const queryParams = new URLSearchParams(this.options.windowRef.location.search);
        let value = {};
        for (const def of this.defs) {
            if (def.strategy === 'twoWay') {
                const queryKey = def.queryKey;
                if (queryParams.has(queryKey)) {
                    set(value, def.path, def.parse(queryParams.get(queryKey)));
                }
            }
        }
        this.group.patchValue(value);
        const onSubmitDefs = this.defs.filter((def) => def.trigger === 'submit');
        const onChangeDefs = this.defs.filter((def) => def.trigger === 'change');
        if (onSubmitDefs.length) {
            this.formGroupDirective.ngSubmit.pipe(takeUntil(this.destroy)).subscribe(() => {
                this.updateQueryParams(resolveParams(onSubmitDefs, this.group.value, this.group), true);
            });
        }
        if (onChangeDefs.length) {
            this.group.valueChanges.pipe(takeUntil(this.destroy)).subscribe((value) => {
                this.updateQueryParams(resolveParams(onChangeDefs, value, this.group));
            });
        }
    }
    get group() {
        return this.formGroupDirective.control;
    }
    ngOnDestroy() {
        this.destroy.next();
    }
    updateQueryParams(queryParams, replaceUrl = false) {
        this.router.navigate([], {
            queryParams,
            queryParamsHandling: 'merge',
            replaceUrl,
        });
    }
}
BindQueryParamsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[bindQueryParams]',
            },] }
];
BindQueryParamsDirective.ctorParameters = () => [
    { type: ControlContainer },
    { type: Router },
    { type: undefined, decorators: [{ type: Inject, args: [BIND_QUERY_PARAMS_OPTIONS,] }] }
];
BindQueryParamsDirective.propDecorators = {
    _defs: [{ type: Input, args: ['bindQueryParams',] }]
};
class BindQueryParamsModule {
    static forRoot(options) {
        return {
            ngModule: BindQueryParamsModule,
            providers: [
                {
                    provide: BIND_QUERY_PARAMS_OPTIONS,
                    useValue: options,
                },
            ],
        };
    }
}
BindQueryParamsModule.decorators = [
    { type: NgModule, args: [{
                exports: [BindQueryParamsDirective],
                declarations: [BindQueryParamsDirective],
            },] }
];

/*
 * Public API Surface of bind-query-params
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BindQueryParamsManager, BindQueryParamsModule, BindQueryParamsDirective as ɵa, BIND_QUERY_PARAMS_OPTIONS as ɵc };
//# sourceMappingURL=ngneat-bind-query-params.js.map
